import 'dart:async';

import 'package:flutter/services.dart';

import 'package:mic_stream/mic_stream.dart';

const AudioSource _DEFAULT_AUDIO_SOURCE = AudioSource.DEFAULT;
const ChannelConfig _DEFAULT_CHANNELS_CONFIG = ChannelConfig.CHANNEL_IN_MONO;
const AudioFormat _DEFAULT_AUDIO_FORMAT = AudioFormat.ENCODING_PCM_8BIT;
const int _DEFAULT_SAMPLE_RATE = 16000;

const int _MIN_SAMPLE_RATE = 1;
const int _MAX_SAMPLE_RATE = 100000;

const EventChannel _microphoneEventChannel =
        EventChannel('aaron.code.com/mic_stream');

class Microphone {
    Microphone({
        AudioSource audioSource: _DEFAULT_AUDIO_SOURCE,
        int sampleRate: _DEFAULT_SAMPLE_RATE,
        ChannelConfig channelConfig: _DEFAULT_CHANNELS_CONFIG,
        AudioFormat audioFormat: _DEFAULT_AUDIO_FORMAT
    }) {
        if (sampleRate < _MIN_SAMPLE_RATE || sampleRate > _MAX_SAMPLE_RATE)
            throw (RangeError.range(sampleRate, _MIN_SAMPLE_RATE, _MAX_SAMPLE_RATE));


         _checkPermissions(
            _microphoneEventChannel.receiveBroadcastStream([
                audioSource.index,
                sampleRate,
                channelConfig == ChannelConfig.CHANNEL_IN_MONO ? 16 : 12,
                audioFormat == AudioFormat.ENCODING_PCM_8BIT ? 3 : 2
            ])).map(_squashList).forEach((element) {
                _transform(element).forEach((element) {
                    _controller.add(element);
                });});
    }

    StreamController <int> _controller = StreamController();

    Stream <int> get stream => _controller.stream;
    Stream <int> get asBroadcast => stream.asBroadcastStream();

    dynamic _checkPermissions(var stream) async* {
        if (!(await permissionStatus))
            throw (PlatformException);

        yield* stream;
    }

    Stream <int> _transform (List <int> data) async* {
        for (var element in data) {
            yield element;
        }
    }

    List<int> _squashList (dynamic byteSamples) {
        List<int> shortSamples = List();
        bool isFirstElement = true;
        int first;
        for (var sample in byteSamples) {
            if (isFirstElement)
                first = sample;
            else
                shortSamples.add(_squashPair(first, sample));
            isFirstElement = !isFirstElement;
        }
        return shortSamples;
    }

    int _squashPair (int first, int second) {
        return first * 256 + second - 32767;
    }

    void dispose() => _controller.close();
}
